@@ -17,7 +17,7 @@
  * with this program; if not, write to the Free Software Foundation, Inc.,
  * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  *
- * (C) Copyright 2008 Red Hat, Inc.
+ * (C) Copyright 2008 - 2011 Red Hat, Inc.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -30,12 +30,47 @@
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
 
+#include <gnome-keyring.h>
+#include <gnome-keyring-memory.h>
+
 #include <nm-setting-vpn.h>
 #include <nm-vpn-plugin-utils.h>
 
 #include "src/nm-pptp-service.h"
-#include "common-gnome/keyring-helpers.h"
-#include "gnome-two-password-dialog.h"
+#include "vpn-password-dialog.h"
+
+#define KEYRING_UUID_TAG "connection-uuid"
+#define KEYRING_SN_TAG "setting-name"
+#define KEYRING_SK_TAG "setting-key"
+
+static char *
+keyring_lookup_secret (const char *uuid, const char *secret_name)
+{
+	GList *found_list = NULL;
+	GnomeKeyringResult ret;
+	GnomeKeyringFound *found;
+	char *secret = NULL;
+
+	ret = gnome_keyring_find_itemsv_sync (GNOME_KEYRING_ITEM_GENERIC_SECRET,
+	                                      &found_list,
+	                                      KEYRING_UUID_TAG,
+	                                      GNOME_KEYRING_ATTRIBUTE_TYPE_STRING,
+	                                      uuid,
+	                                      KEYRING_SN_TAG,
+	                                      GNOME_KEYRING_ATTRIBUTE_TYPE_STRING,
+	                                      NM_SETTING_VPN_SETTING_NAME,
+	                                      KEYRING_SK_TAG,
+	                                      GNOME_KEYRING_ATTRIBUTE_TYPE_STRING,
+	                                      secret_name,
+	                                      NULL);
+	if (ret == GNOME_KEYRING_RESULT_OK && found_list) {
+		found = g_list_nth_data (found_list, 0);
+		secret = gnome_keyring_memory_strdup (found->secret);
+	}
+
+	gnome_keyring_found_list_free (found_list);
+	return secret;
+}
 
 static gboolean
 get_secrets (const char *vpn_uuid,
@@ -46,9 +81,9 @@ get_secrets (const char *vpn_uuid,
              char **out_pw,
              NMSettingSecretFlags pw_flags)
 {
-	GnomeTwoPasswordDialog *dialog;
-	gboolean is_session = TRUE;
+	VpnPasswordDialog *dialog;
 	char *prompt, *pw = NULL;
+	const char *new_password = NULL;
 
 	g_return_val_if_fail (vpn_uuid != NULL, FALSE);
 	g_return_val_if_fail (vpn_name != NULL, FALSE);
@@ -61,7 +96,7 @@ get_secrets (const char *vpn_uuid,
 		if (in_pw)
 			pw = gnome_keyring_memory_strdup (in_pw);
 		else
-			pw = keyring_helpers_lookup_secret (vpn_uuid, NM_PPTP_KEY_PASSWORD, &is_session);
+			pw = keyring_lookup_secret (vpn_uuid, NM_PPTP_KEY_PASSWORD);
 	}
 
 	/* Don't ask if the passwords is unused */
@@ -90,56 +125,22 @@ get_secrets (const char *vpn_uuid,
 	 * that the password should never be saved.
 	 */
 	prompt = g_strdup_printf (_("You need to authenticate to access the Virtual Private Network '%s'."), vpn_name);
-	dialog = GNOME_TWO_PASSWORD_DIALOG (gnome_two_password_dialog_new (_("Authenticate VPN"), prompt, NULL, NULL, FALSE));
+	dialog = (VpnPasswordDialog *) vpn_password_dialog_new (_("Authenticate VPN"), prompt, NULL);
 	g_free (prompt);
 
-	gnome_two_password_dialog_set_show_username (dialog, FALSE);
-	gnome_two_password_dialog_set_show_userpass_buttons (dialog, FALSE);
-	gnome_two_password_dialog_set_show_domain (dialog, FALSE);
-	gnome_two_password_dialog_set_show_remember (dialog, TRUE);
-	gnome_two_password_dialog_set_show_password_secondary (dialog, FALSE);
-
-	/* If nothing was found in the keyring, default to not remembering any secrets */
-	if (pw) {
-		/* Otherwise set default remember based on which keyring the secrets were found in */
-		if (is_session)
-			gnome_two_password_dialog_set_remember (dialog, GNOME_TWO_PASSWORD_DIALOG_REMEMBER_SESSION);
-		else
-			gnome_two_password_dialog_set_remember (dialog, GNOME_TWO_PASSWORD_DIALOG_REMEMBER_FOREVER);
-	} else
-		gnome_two_password_dialog_set_remember (dialog, GNOME_TWO_PASSWORD_DIALOG_REMEMBER_NOTHING);
+	vpn_password_dialog_set_show_password_secondary (dialog, FALSE);
 
 	/* pre-fill dialog with the password */
 	if (pw && !(pw_flags & NM_SETTING_SECRET_FLAG_NOT_SAVED))
-		gnome_two_password_dialog_set_password (dialog, pw);
+		vpn_password_dialog_set_password (dialog, pw);
 
 	gtk_widget_show (GTK_WIDGET (dialog));
 
-	if (gnome_two_password_dialog_run_and_block (dialog)) {
-		const char *keyring = NULL;
-		gboolean save = FALSE;
+	if (vpn_password_dialog_run_and_block (dialog)) {
 
-		*out_pw = gnome_two_password_dialog_get_password (dialog);
-
-		switch (gnome_two_password_dialog_get_remember (dialog)) {
-		case GNOME_TWO_PASSWORD_DIALOG_REMEMBER_SESSION:
-			keyring = "session";
-			/* Fall through */
-		case GNOME_TWO_PASSWORD_DIALOG_REMEMBER_FOREVER:
-			save = TRUE;
-			break;
-		default:
-			break;
-		}
-
-		if (save && (pw_flags & NM_SETTING_SECRET_FLAG_AGENT_OWNED)) {
-		    if (*out_pw && !(pw_flags & NM_SETTING_SECRET_FLAG_NOT_SAVED))
-				keyring_helpers_save_secret (vpn_uuid, vpn_name, keyring, NM_PPTP_KEY_PASSWORD, *out_pw);
-			else {
-				/* Clear the password from the keyring */
-				keyring_helpers_delete_secret (vpn_uuid, NM_PPTP_KEY_PASSWORD);
-			}
-		}
+		new_password = vpn_password_dialog_get_password (dialog);
+		if (new_password)
+			*out_pw = gnome_keyring_memory_strdup (new_password);
 	}
 
 	gtk_widget_hide (GTK_WIDGET (dialog));
@@ -230,10 +231,8 @@ main (int argc, char *argv[])
 		printf ("%s\n%s\n", NM_PPTP_KEY_PASSWORD, password);
 	printf ("\n\n");
 
-	if (password) {
-		memset (password, 0, strlen (password));
+	if (password)
 		gnome_keyring_memory_free (password);
-	}
 
 	/* for good measure, flush stdout since Kansas is going Bye-Bye */
 	fflush (stdout);
